@@ -4,6 +4,11 @@
 #undef CONF_CHECK_HTTPD_USER
 #undef CONF_HTTPD_USER 
 
+/*
+ * Chroot prefix, requires significant infrastructure to use
+ */
+#undef CONF_CHROOT_PREFIX
+
 /*
  * Informational definitions
  */
@@ -48,6 +53,7 @@
 #undef HAS_SIGSET
 #undef HAS_INITGROUPS
 #undef HAS_SETGROUPS
+#undef HAS_GETGROUPS
 #undef HAS_SETGID
 #undef HAS_SETUID
 #undef HAS_SETEGID
@@ -70,7 +76,7 @@
  * Various options
  */
 #undef CONF_MINIMUM_UID
-#undef CONF_FIXED_PATHTRANS
+#undef CONF_MINIMUM_GID
 #undef CONF_REDIR_STDERR
 #undef CONF_SUBDIRS
 #undef CONF_USERDIRFILE
@@ -82,6 +88,7 @@
 #undef CONF_CHECKSHELL
 #undef CONF_AFS_SETPAG
 #undef CONF_CGIDIR
+#undef CONF_MULTIUSER_CGI_DIR
 
 /*
  * RLimit Options
@@ -108,3 +115,9 @@
  * OS Specific Definitions
  */
 #undef _HPUX_SOURCE
+
+/*
+ * Quiet warning about rcsid
+ */
+#define RCSID(msg) \
+static /**/const char *const rcsid[] = { (char *)rcsid, "\100(#)" msg }
