@@ -1,11 +1,93 @@
-static char *rcsid="$Id: util.c,v 1.42 1999/04/21 15:46:51 nneul Exp $";
-
 /**
  **  File: util.c
  **  Purpose: Various utility routines used by cgiwrap
  **/ 
 
 #include "cgiwrap.h"	/* Headers for all CGIwrap source files */
+RCSID("$Id: util.c,v 1.50 2001/10/30 15:10:15 nneul Exp $");
+
+/*
+ * Encode string to protect against cross-site scripting, but only
+ * encode if we are outputting in HTML. Otherwise send as is so that
+ * error message shows up as transmitted.
+ *
+ * Strings is returned in malloced space, but should be treated as if
+ * it was allocated staticly.
+ */
+char *HTMLEncode(char *what)
+{
+	static char *res = NULL;
+	static int len = 0;
+	int i,j;
+
+	/* passed a null string, so return same */
+	if ( !what )
+	{
+		return NULL;	
+	}
+
+	/* if we are in plaintext mode, just return string as is */
+	if ( ! MSG_HTMLMessages )
+	{
+		return what;
+	}
+
+	/* deallocate previous mem if not sufficient for current string */
+	/* largest encoding is &amp; so *5 should be sufficient. */
+	if ( res && ((strlen(what)*5+1) > len) )
+	{
+		free(res);
+		res = NULL;
+		len = 0;
+	}
+
+	/* allocate mem */
+	if ( ! res )
+	{
+		/* default to HUGE_STRING_LEN to eliminate excess mallocing */
+		len = (strlen(what)*5+1);
+		if ( len < HUGE_STRING_LEN )
+		{
+			len = HUGE_STRING_LEN;
+		}
+		res = SafeMalloc(len, "SafeEncode");
+	}
+
+	/* copy string to result, encoding as we go */
+	j = 0;
+	for ( i = 0; i<=strlen(what); i++ )
+	{
+		if ( what[i] == '<' )
+		{
+			res[j++] = '&';
+			res[j++] = 'l';
+			res[j++] = 't';
+			res[j++] = ';';
+		}
+		else if ( what[i] == '>' )
+		{
+			res[j++] = '&';
+			res[j++] = 'g';
+			res[j++] = 't';
+			res[j++] = ';';
+		}
+		else if ( what[i] == '&' )
+		{
+			res[j++] = '&';
+			res[j++] = 'a';
+			res[j++] = 'm';
+			res[j++] = 'p';
+			res[j++] = ';';
+		}
+		else
+		{
+			res[j++] = what[i];
+		}
+	}
+	res[j] = '\0';
+
+	return res;
+}
 
 /*
  * Safe malloc
@@ -175,6 +257,7 @@ void OutputEnvironment(void)
  	DEBUG_Msg("Environment Variables:");  
 	DEBUG_Str("     QUERY_STRING:", (char *) getenv("QUERY_STRING") );
         DEBUG_Str("      SCRIPT_NAME:", (char *) getenv("SCRIPT_NAME") );
+        DEBUG_Str("  SCRIPT_FILENAME:", (char *) getenv("SCRIPT_FILENAME") );
         DEBUG_Str("        PATH_INFO:", (char *) getenv("PATH_INFO") );
         DEBUG_Str("  PATH_TRANSLATED:", (char *) getenv("PATH_TRANSLATED") );
         DEBUG_Str("      REMOTE_USER:", (char *) getenv("REMOTE_USER") );
@@ -240,12 +323,14 @@ void CheckUser(struct passwd *user)
 #if defined(CONF_DENYFILE)
 	if ( deny_exists )
 	{
+	        DEBUG_Str("Checking deny file for",user->pw_name);
 		in_deny = UserInFile(CONF_DENYFILE, user->pw_name);
 	}
 #endif
 #if defined(CONF_ALLOWFILE)
 	if ( allow_exists )
 	{
+	        DEBUG_Str("Checking allow file for",user->pw_name);
 		in_allow = UserInFile(CONF_ALLOWFILE, user->pw_name);
 	}
 #endif
@@ -282,6 +367,14 @@ void CheckUser(struct passwd *user)
 	}
 #endif
 
+#if defined(CONF_MINIMUM_GID)
+	if ( user->pw_gid < CONF_MINIMUM_GID )
+	{
+		Log(user->pw_name, "-", "gid less than minimum");
+		MSG_Error_AccessControl("GID of script userid less than configured minimum.",NULL);
+	}
+#endif
+
 #if defined(CONF_CHECKSHELL)
 	{
 	char *sh, *getusershell();
@@ -338,20 +431,20 @@ void CheckScriptFile(void)
 			"Script file not found.");
 	}
 
-#if defined(CONF_CHECK_SYMLINK)
 	if ( lstat(Context.scriptFullPath, &fileLStat) )
 	{
 		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
 			"Script file not found.");
 	}
 
+#if defined(CONF_CHECK_SYMLINK)
 	if ( S_ISLNK(fileLStat.st_mode) )
 	{
 		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
 			"Script is a symbolic link");
 	}
 #endif		
-	
+
 	if ( !S_ISREG(fileStat.st_mode) )
 	{
 		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
@@ -366,16 +459,27 @@ void CheckScriptFile(void)
 
 
 #if defined(CONF_CHECK_SCRUID)
-	if (fileStat.st_uid != Context.user.pw_uid)
+	if (!S_ISLNK(fileLStat.st_mode))
 	{
-		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
-			"Script does not have same UID");
+		if (!Context.multiuser_cgi_script && fileStat.st_uid != Context.user.pw_uid)
+		{
+			MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
+				"Script does not have same UID");
+		}
+	}
+	else
+	{
+		if (!Context.multiuser_cgi_script && fileLStat.st_uid != Context.user.pw_uid)
+		{
+			MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
+				"Symlink to script does not have same UID");
+		}
 	}
 #endif
 
 
 #if defined(CONF_CHECK_SCRGID)
-	if (fileStat.st_gid != Context.user.pw_gid)
+	if (!Context.multiuser_cgi_script && fileStat.st_gid != Context.user.pw_gid)
 	{
 		MSG_Error_ExecutionNotPermitted(Context.scriptRelativePath,
 			"Script does not have same GID");
@@ -733,6 +837,11 @@ void ChangeID ( struct passwd *user)
  */
 void ChangeAuxGroups(struct passwd *user)
 {
+#if defined(CONF_MINIMUM_GID) && defined(HAS_GETGROUPS)
+	int i,j;
+	gid_t *groups;
+#endif
+
 #if defined(HAS_SETGROUPS) && defined(CONF_SETGROUPS)
 	gid_t grouplist[1];
 	grouplist[0] = user->pw_gid;
@@ -745,6 +854,26 @@ void ChangeAuxGroups(struct passwd *user)
 	if ( initgroups( user->pw_name, user->pw_gid ) == -1 )
 		MSG_Error_SystemError("initgroups() failed!");
 #endif
+
+	/* verify here that we didn't enable a group less than minimum */
+#if defined(CONF_MINIMUM_GID) && defined(HAS_GETGROUPS)
+	i = getgroups(0, groups);
+	if ( i > 0 )
+	{
+		groups = (gid_t *)SafeMalloc(i * sizeof(gid_t), "Auxilliary Group List");
+		getgroups(i, groups);
+
+		for ( j=0; j<i; j++ )
+		{
+printf("checking %d against %d\n", groups[j], CONF_MINIMUM_GID);
+			if ( groups[j] < CONF_MINIMUM_GID )
+			{
+                		Log(user->pw_name, "-", "supplementary gid less than minimum");
+                		MSG_Error_AccessControl("Supplementary GID of script userid less than configured minimum.",NULL);
+			}
+		}
+	}
+#endif
 }
 
 
@@ -759,7 +888,7 @@ int UserInFile(char *filename, char *user)
 	int count, remote_addr[4],spec_mask[4],spec_addr[4];
 	char *i;
 #endif
-	int j;
+	int j, intail;
 
 #if defined(CONF_CHECKHOST)
 	if (sscanf(getenv("REMOTE_ADDR"),"%d.%d.%d.%d",
@@ -784,17 +913,33 @@ int UserInFile(char *filename, char *user)
 			fclose(file);
 			return 0;
 		}
-		for (j=0; j<=strlen(temp); j++)
+		intail=1;
+		for (j=strlen(temp)-1; j>=0; j--)
 		{
 			if ( !isprint(temp[j]) )
 			{
 				temp[j] = 0;
 			}
+			else 
+                        { 
+                             if (intail) 
+			     {
+			           if (isspace(temp[j]))
+				   {
+				         temp[j] = 0;
+				   }
+				   else 
+				   {
+				         intail = 0;
+				   }
+			     }
+			}
 		}
 
 		if ( !strcmp(temp,user) )
 		{
 			fclose(file);
+			DEBUG_Str("Found",user);
 			return 1;
 		}
 #if defined(CONF_CHECKHOST)
@@ -941,37 +1086,55 @@ void LogEnd(void)
 void SetScriptName(char *userStr, char *scrStr )
 {
 	char *buf;
+	char *name;
 
-	buf = (char*) SafeMalloc (strlen("SCRIPT_NAME") +
-	    strlen(getenv("SCRIPT_NAME")) + strlen(userStr)
-	    + strlen(scrStr) + 5, "new SCRIPT_NAME environment variable");
+	name = getenv("SCRIPT_NAME");
+	if ( name ) {
+		/* only set SCRIPT_NAME if it was already set */
 
-	sprintf(buf, "%s=%s/%s/%s", "SCRIPT_NAME", 
-	    getenv("SCRIPT_NAME"), userStr, scrStr);
-	SafePutenv(buf, "set SCRIPT_NAME environment variable");
+		buf = (char*) SafeMalloc (strlen("SCRIPT_NAME") +
+		    strlen(name) + strlen(userStr)
+		    + strlen(scrStr) + 5, "new SCRIPT_NAME environment variable");
+
+		sprintf(buf, "%s=%s/%s/%s", "SCRIPT_NAME", 
+		    name, userStr, scrStr);
+		SafePutenv(buf, "set SCRIPT_NAME environment variable");
+	}
 }
 
 
+/*
+ * Set the correct SCRIPT_FILENAME environment variable (PHP uses this)
+ */
+void SetScriptFilename (char *scriptPath) 
+{
+	char *buf;
+  char *name;
+
+	name = getenv ("SCRIPT_FILENAME");
+	if ( name ) {
+		/* only set SCRIPT_FILENAME if it was already set */
+
+		buf = (char*) SafeMalloc (strlen("SCRIPT_FILENAME") +
+		    + strlen(scriptPath) + 5, "new SCRIPT_FILENAME environment variable");
+
+		sprintf(buf, "%s=%s", "SCRIPT_FILENAME", scriptPath);
+		SafePutenv(buf, "set SCRIPT_NAME environment variable");
+	}
+}
+
+
+
 /*
  * Set the correct PATH_TRANSLATED environment variable
  */
-void SetPathTranslated( char *scriptPath )
+void SetPathTranslated( char *cgiBaseDir, char *scriptPath )
 {
-#ifndef CONF_FIXED_PATHTRANS
-	char *buf;
-	buf = (char *) SafeMalloc( strlen("PATH_TRANSLATED") +
-		strlen(scriptPath) + 5, "new PATH_TRANSLATED environment variable");
-
-	sprintf(buf, "%s=%s", "PATH_TRANSLATED", scriptPath); 
-	SafePutenv(buf, "set PATH_TRANSLATED environment variable");
-
-#else
-	char *buf, *docroot, *pathinfo;
+	char *buf, *pathinfo;
 	
-	docroot = getenv("DOCUMENT_ROOT");
 	pathinfo = getenv("PATH_INFO");
 
-	if ( !docroot || !pathinfo )
+	if ( !pathinfo )
 	{
 		SafePutenv("PATH_TRANSLATED=", 
 			"set PATH_TRANSLATED environment variable");
@@ -979,13 +1142,11 @@ void SetPathTranslated( char *scriptPath )
 	}
 
 	buf = (char *) SafeMalloc( strlen("PATH_TRANSLATED") +
-		strlen(docroot) + strlen(pathinfo) + 5, 
+		strlen(cgiBaseDir) + strlen(pathinfo) + 5, 
 		"new PATH_TRANSLATED environment variable");
 
-	sprintf(buf, "%s=%s%s", "PATH_TRANSLATED", docroot, pathinfo); 
+	sprintf(buf, "%s=%s%s", "PATH_TRANSLATED", cgiBaseDir, pathinfo); 
 	putenv(buf);
-#endif
-
 }
 
 
