@@ -1,10 +1,11 @@
-/* rcsid: $Id: util.h,v 1.16 1999/04/14 20:15:14 nneul Exp $ */
+/* rcsid: $Id: util.h,v 1.19 2001/07/22 17:22:07 nneul Exp $ */
 
 /**
  **  File: util.h
  **  Purpose: Prototypes and defines for routines in util.c
  **/ 
 
+char *HTMLEncode( char *what );
 char *SafeMalloc( size_t size, char *what );
 void SafePutenv( char * string, char *what );
 char **CreateARGV( char *scrStr, int argc, char *argv[]);
@@ -33,7 +34,8 @@ void ChangeToCGIDir(char *scriptPath);
 void DoError (char *msg);
 void Log (char *user, char *script, char *msg);
 void SetScriptName(char *userStr, char *scrStr );
-void SetPathTranslated(char *scriptPath);
+void SetScriptFilename(char *scriptPath );
+void SetPathTranslated(char *cgiBaseDir, char *scriptPath);
 void Create_AFS_PAG(void);
 char *GetUserDir(char *user);
 char *GetBaseDirectory(struct passwd *user);
